.PHONY: cover-fixup

IMAGE_OPTIONS ?=

cover-fixup: ## run mocha unit tests with coverage reports
cover-fixup: cover
	sed -i $(TARGET_PATH)/coverage/lcov.info \
		-e '/external ".*"$$/,/end_of_record/d' \
		-e '/ sync$$/,/end_of_record/d' \
		-e '/test sync/,/end_of_record/d' \
		-e '/node_modules/,/end_of_record/d' \
		-e '/bootstrap$$/,/end_of_record/d' \
		-e '/universalModuleDefinition/,/end_of_record/d'
	sed -n '/^SF/,$$p' -i $(TARGET_PATH)/coverage/lcov.info
	sed '1s;^;TN:\n;' -i $(TARGET_PATH)/coverage/lcov.info

full: ## ultra thorough build (looong)
	$(MAKE) clean-target ci
	$(MAKE) clean-target local-alpine local-chown-leaks
	$(MAKE) clean-target local-stretch local-chown-leaks
	$(MAKE) clean-target
	@echo "Full build (CI, alpine, stretch) succeeded!"

# run targets
run: run-app

run-app: ## run the app
	@node out/src/index.js

run-debug: ## run the app and wait for an inspector to connect
	@node --inspect-brk --inspect-port=0.0.0.0:9229 out/src/index.js

upload-climate:
	cc-test-reporter format-coverage -t lcov -o $(TARGET_PATH)/coverage/codeclimate.json -p $(ROOT_PATH) $(TARGET_PATH)/coverage/lcov.info
	cc-test-reporter upload-coverage --debug -i $(TARGET_PATH)/coverage/codeclimate.json -r "$(shell echo "${CODECLIMATE_SECRET}" | base64 -d)"

run-assembly:
	node out/src/index.js assembly ./src/examples/loop.assembly.ssrp

run-abstract:
	node out/src/index.js abstract ./src/examples/loop.abstract.ssrp