import { readFile } from 'node:fs/promises';
import { argv } from 'node:process';

import { codec as abstractCodec } from './module/abstract.js';
import { codec as assemblyCodec } from './module/assembly.js';
import { codec as binaryCodec } from './module/binary.js';
import { Codec } from './module/index.js';
import { runProgram } from './runtime.js';

const CODECS: Record<string, Codec> = {
  assembly: assemblyCodec,
  abstract: abstractCodec,
  binary: binaryCodec,
};

async function main() {
  const [_node, _script, format, sourcePath] = argv;
  console.log('loading', sourcePath);

  const source = await readFile(sourcePath, {
    encoding: 'utf-8',
  });
  const main = CODECS[format].load(source);

  const stack = runProgram(main);
  console.log('stack', stack);
}

main().then(() => process.exit(0)).catch((err) => {
  console.error('uncaught error', err);
  process.exit(1);
});
