import pegjs from 'pegjs';

import { FunctionNode, RawRegisters } from '../ast.js';
import { loadFile } from '../files.js';
import { Codec, ProgramModule } from './index.js';

const { generate } = pegjs;

export function loadGrammar() {
  const grammar = loadFile('src', 'module', 'abstract.peg');
  return generate(grammar);
}

export interface ParserOutput {
  functions: Array<FunctionNode>;
  registers: Partial<RawRegisters>;
}

export function abstractToAST(source: string): ProgramModule {
  const parser = loadGrammar();
  const ast = parser.parse(source) as ParserOutput;
  console.log(ast);

  return {
    functions: {
      names: [['main', 0]],
      entries: ast.functions,
    },
    globals: {
      names: [],
      entries: [],
    },
    registers: ast.registers,
  };
}

export function abstractFromAST(module: ProgramModule): string {
  throw new Error('not implemented');
}

export const codec: Codec = {
  load: abstractToAST,
  save: abstractFromAST,
};
