import pegjs from 'pegjs';

import { CommandNode, RawRegisters } from '../ast.js';
import { loadFile } from '../files.js';
import { Codec, ProgramModule } from './index.js';

const { generate } = pegjs;

export function loadGrammar() {
  const grammar = loadFile('src', 'module', 'assembly.peg');
  return generate(grammar);
}

export interface ParserOutput {
  instructions: Array<CommandNode>;
  registers: Partial<RawRegisters>;
}

/**
 * Load a module from assembly text.
 */
export function assemblyToAST(data: string): ProgramModule {
  const parser = loadGrammar();
  const ast = parser.parse(data) as ParserOutput;
  console.log(ast);

  return {
    functions: {
      names: [['main', 0]],
      entries: [{
        type: 'function',
        body: ast.instructions,
      }],
    },
    globals: {
      names: [],
      entries: [],
    },
    registers: ast.registers,
  };
}

export function assemblyFromAST(module: ProgramModule): string {
  throw new Error('not implemented');
}

export const codec: Codec = {
  load: assemblyToAST,
  save: assemblyFromAST,
};
