import { Address, ProgramRegisters, RawRegisters, Register } from './ast.js';
import { Program, ProgramModule } from './module/index.js';
import { RUNTIME_MODULE } from './runtime/module/base.js';
import { step } from './runtime/node.js';

export function address(offset = 0): Address {
  return {
    type: 'address',
    module: 0,
    function: 0,
    offset,
  };
}

export function register(value = 0): Register {
  return {
    type: 'data',
    value,
  };
}

export function completeRegisters(partial: Partial<RawRegisters>): ProgramRegisters {
  return {
    PC: address(partial.PC),
    RA: address(partial.RA),
    EH: address(partial.EH),
    IH: address(partial.IH),
    R1: register(partial.R1),
    R2: register(partial.R2),
    R3: register(partial.R3),
    R4: register(partial.R4),
  };
}

/**
 * Prepare the memory for a program module to run.
 */
export function prep(runtime: ProgramModule, program: ProgramModule): Program {
  return {
    registers: completeRegisters(program.registers),
    runtime,
    program,
    modules: [],
    stack: [],
  };
}

/**
 * Step the program until the function has been exhausted.
 */
export function runProgram(main: ProgramModule): Array<number> {
  const state = prep(RUNTIME_MODULE, main);
  console.log('start');
  const fn = state.program.functions.entries[0];
  while (state.registers.PC.offset < fn.body.length) {
    if (step(state, fn) === true) {
      break;
    }
  }
  console.log('exit');
  return state.stack;
}
