import { CommandNode, OpCode, ProgramRegisters } from '../../ast.js';
import { ProgramStack } from '../../module/index.js';
import { RUNTIME_FUNCTIONS } from '../module/base.js';
import { getValue, setValue } from '../register.js';
import { OpCodeFn, StepResult } from './base.js';

export const FLOW_CODES: Partial<Record<OpCode, OpCodeFn>> = {
  [OpCode.CALL]: opCall,
  [OpCode.JUMP]: opJump,
  [OpCode.STOP]: opStop,
};

export function opCall(instruction: CommandNode, registers: ProgramRegisters, stack: ProgramStack): StepResult {
  if (instruction.second && getValue(registers, instruction.first) == 0) {
    const fn = RUNTIME_FUNCTIONS[getValue(registers, instruction.second)];
    fn(stack);
    return {
      stop: false,
      step: 0,
    };
  }

  const dest = getValue(registers, instruction.first);
  console.log('CALL', dest);
  registers.PC.offset = dest;
  return {
    stop: false,
    step: 0,
  };
}

export function opJump(instruction: CommandNode, registers: ProgramRegisters, stack: ProgramStack): StepResult {
  const dest = getValue(registers, instruction.first);
  console.log('JUMP', dest);
  setValue(registers, 'PC', dest);
  return {
    stop: false,
    step: 0,
  };
}

export function opStop(instruction: CommandNode, registers: ProgramRegisters, stack: ProgramStack): StepResult {
  return {
    stop: true,
    step: 0,
  };
}
