import { CommandNode, OpCode, ProgramRegisters } from '../../ast.js';
import { ProgramStack } from '../../module/index.js';
import { getRegister, setValue, getValue } from '../register.js';
import { OpCodeFn, StepResult } from './base.js';

export const STACK_CODES: Partial<Record<OpCode, OpCodeFn>> = {
  [OpCode.PEEK]: opPeek,
  [OpCode.POP]: opPop,
  [OpCode.PUSH]: opPush,
};

export function opPeek(instruction: CommandNode, registers: ProgramRegisters, stack: ProgramStack): StepResult {
  const reg = getRegister(instruction.first);
  console.log('PEEK', reg);
  setValue(registers, reg, stack[stack.length - 1] || 0);
  return {
    step: 1,
    stop: false,
  };
}

export function opPush(instruction: CommandNode, registers: ProgramRegisters, stack: ProgramStack): StepResult {
  const val = getValue(registers, instruction.first);
  console.log('PUSH', val);
  stack.push(val);
  return {
    step: 1,
    stop: false,
  };
}

export function opPop(instruction: CommandNode, registers: ProgramRegisters, stack: ProgramStack): StepResult {
  const reg = getRegister(instruction.first);
  console.log('POP', reg);
  setValue(registers, reg, stack.pop() || 0);
  return {
    step: 1,
    stop: false,
  };
}
