import { ProgramRegisters, RegisterName, ValueNode } from '../ast.js';

export function getRegister(value?: ValueNode): RegisterName {
  if (value === null || value === undefined) {
    throw new Error('missing value');
  }

  if (value.type === 'register') {
    return value.register;
  } else {
    throw new Error('invalid value');
  }
}

export function getValue(registers: ProgramRegisters, value?: ValueNode): number {
  if (value === null || value === undefined) {
    return 0;
  }

  if (value.type === 'register') {
    const register = registers[value.register];
    if (register.type === 'address') {
      return register.offset;
    } else {
      return register.value;
    }
  } else {
    return value.constant;
  }
}

export function setValue(registers: ProgramRegisters, name: RegisterName, value: number): void {
  const register = registers[name];
  if (register.type === 'address') {
    register.offset = value;
  } else {
    register.value = value;
  }
}
